@@ -84,7 +84,10 @@ namespace KisLayerUtils {
         QSet<int> frames;
 
         virtual KisNodeList allSrcNodes() = 0;
-        virtual KisLayerSP dstLayer() { return 0; }
+
+        KisLayerSP dstLayer() {
+            return qobject_cast<KisLayer*>(dstNode.data());
+        }
     };
 
     struct MergeDownInfo : public MergeDownInfoBase {
@@ -110,7 +113,7 @@ namespace KisLayerUtils {
             return mergedNodes;
         }
 
-        KisLayerSP dstLayer() override {
+        KisLayerSP dstLayer() {
             return dynamic_cast<KisLayer*>(dstNode.data());
         }
     };
@@ -197,6 +200,40 @@ namespace KisLayerUtils {
         MergeDownInfoBaseSP m_info;
     };
 
+    struct DisablePassThroughForHeadsOnly : public KisCommandUtils::AggregateCommand {
+        DisablePassThroughForHeadsOnly(MergeDownInfoBaseSP info, bool skipIfDstIsGroup = false)
+            : m_info(info),
+              m_skipIfDstIsGroup(skipIfDstIsGroup)
+        {
+        }
+
+        void populateChildCommands() override {
+            if (m_skipIfDstIsGroup &&
+                m_info->dstLayer() &&
+                m_info->dstLayer()->inherits("KisGroupLayer")) {
+
+                return;
+            }
+
+
+            Q_FOREACH (KisNodeSP node, m_info->allSrcNodes()) {
+                if (KisLayerPropertiesIcons::nodeProperty(node, KisLayerPropertiesIcons::passThrough, false).toBool()) {
+
+                    KisBaseNode::PropertyList props = node->sectionModelProperties();
+                    KisLayerPropertiesIcons::setNodeProperty(&props,
+                                                             KisLayerPropertiesIcons::passThrough,
+                                                             false);
+
+                    addCommand(new KisNodePropertyListCommand(node, props));
+                }
+            }
+        }
+
+    private:
+        MergeDownInfoBaseSP m_info;
+        bool m_skipIfDstIsGroup;
+    };
+
     struct RefreshHiddenAreas : public KUndo2Command {
         RefreshHiddenAreas(MergeDownInfoBaseSP info) : m_info(info) {}
 
@@ -306,13 +343,13 @@ namespace KisLayerUtils {
     };
 
     struct CreateMergedLayerMultiple : public KisCommandUtils::AggregateCommand {
-        CreateMergedLayerMultiple(MergeMultipleInfoSP info, const QString name = QString() ) 
+        CreateMergedLayerMultiple(MergeMultipleInfoSP info, const QString name = QString() )
             : m_info(info),
               m_name(name) {}
 
         void populateChildCommands() override {
             QString mergedLayerName;
-            
+
             if (m_name.isEmpty()){
                 const QString mergedLayerSuffix = i18n("Merged");
                 mergedLayerName = m_info->mergedNodes.first()->name();
@@ -324,7 +361,7 @@ namespace KisLayerUtils {
             } else {
                 mergedLayerName = m_name;
             }
-                
+
             m_info->dstNode = new KisPaintLayer(m_info->image, mergedLayerName, OPACITY_OPAQUE_U8);
 
             if (m_info->frames.size() > 0) {
@@ -547,14 +584,14 @@ namespace KisLayerUtils {
     struct InsertNode : public KisCommandUtils::AggregateCommand {
         InsertNode(MergeDownInfoBaseSP info, KisNodeSP putAfter)
             : m_info(info), m_putAfter(putAfter) {}
-        
+
         void populateChildCommands() override {
             addCommand(new KisImageLayerAddCommand(m_info->image,
                                                            m_info->dstNode,
                                                            m_putAfter->parent(),
                                                            m_putAfter,
                                                            true, false));
-        
+
         }
 
     private:
@@ -620,9 +657,14 @@ namespace KisLayerUtils {
                                                            true, false));
                 }
 
-                reparentSelectionMasks(m_info->image,
-                                       m_info->dstLayer(),
-                                       m_info->selectionMasks);
+                /**
+                 * We can merge selection masks, in this case dstLayer is not defined!
+                 */
+                if (m_info->dstLayer()) {
+                    reparentSelectionMasks(m_info->image,
+                                           m_info->dstLayer(),
+                                           m_info->selectionMasks);
+                }
 
                 safeRemoveMultipleNodes(m_info->allSrcNodes(), m_info->image);
             }
@@ -639,9 +681,13 @@ namespace KisLayerUtils {
                                     KisLayerSP newLayer,
                                     const QVector<KisSelectionMaskSP> &selectionMasks) {
 
+            KIS_SAFE_ASSERT_RECOVER_RETURN(newLayer);
+
             foreach (KisSelectionMaskSP mask, selectionMasks) {
-                addCommand(new KisImageLayerMoveCommand(image, mask, newLayer, newLayer->lastChild()));
-                addCommand(new KisActivateSelectionMaskCommand(mask, false));
+                if (mask) {
+                    addCommand(new KisImageLayerMoveCommand(image, mask, newLayer, newLayer->lastChild()));
+                    addCommand(new KisActivateSelectionMaskCommand(mask, false));
+                }
             }
         }
     private:
@@ -773,6 +819,11 @@ namespace KisLayerUtils {
             applicator.applyCommand(new FillSelectionMasks(info));
             applicator.applyCommand(new CreateMergedLayer(info), KisStrokeJobData::BARRIER);
 
+            // in two-layer mode we disable pass trhough only when the destination layer
+            // is not a group layer
+            applicator.applyCommand(new DisablePassThroughForHeadsOnly(info, true));
+            applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
+
             if (info->frames.size() > 0) {
                 foreach (int frame, info->frames) {
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, false, info->storage));
@@ -1016,8 +1067,8 @@ namespace KisLayerUtils {
         applicator.end();
     }
 
-    void mergeMultipleLayersImpl(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter, 
-                                           bool flattenSingleLayer, const KUndo2MagicString &actionName, 
+    void mergeMultipleLayersImpl(KisImageSP image, KisNodeList mergedNodes, KisNodeSP putAfter,
+                                           bool flattenSingleLayer, const KUndo2MagicString &actionName,
                                            bool cleanupNodes = true, const QString layerName = QString())
     {
         filterMergableNodes(mergedNodes);
@@ -1059,6 +1110,7 @@ namespace KisLayerUtils {
             // paint layers and wait until update is finished with a barrier
             applicator.applyCommand(new DisableColorizeKeyStrokes(info));
             applicator.applyCommand(new DisableOnionSkins(info));
+            applicator.applyCommand(new DisablePassThroughForHeadsOnly(info));
             applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
 
             applicator.applyCommand(new KeepMergedNodesSelected(info, putAfter, false));
@@ -1101,7 +1153,7 @@ namespace KisLayerUtils {
     {
         mergeMultipleLayersImpl(image, mergedNodes, putAfter, false, kundo2_i18n("Merge Selected Nodes"));
     }
-    
+
     void newLayerFromVisible(KisImageSP image, KisNodeSP putAfter)
     {
         KisNodeList mergedNodes;
