@@ -30,8 +30,10 @@
 #include <QCloseEvent>
 #include <QDesktopServices>
 #include <QDesktopWidget>
+#include <QDialog>
 #include <QDockWidget>
 #include <QIcon>
+#include <QInputDialog>
 #include <QLabel>
 #include <QLayout>
 #include <QMdiArea>
@@ -64,6 +66,8 @@
 #include <khelpmenu.h>
 #include <klocalizedstring.h>
 #include <kaboutdata.h>
+#include <kis_workspace_resource.h>
+#include <input/kis_input_manager.h>
 
 #ifdef HAVE_KIO
 #include <krecentdocument.h>
@@ -94,6 +98,7 @@
 #include <KoDockRegistry.h>
 #include <KoPluginLoader.h>
 #include <KoColorSpaceEngine.h>
+#include <KoResourceModel.h>
 
 #include <KisMimeDatabase.h>
 #include <brushengine/kis_paintop_settings.h>
@@ -169,10 +174,10 @@ public:
         , dockWidgetMenu(new KActionMenu(i18nc("@action:inmenu", "&Dockers"), parent))
         , windowMenu(new KActionMenu(i18nc("@action:inmenu", "&Window"), parent))
         , documentMenu(new KActionMenu(i18nc("@action:inmenu", "New &View"), parent))
+        , workspaceMenu(new KActionMenu(i18nc("@action:inmenu", "Wor&kspace"), parent))
         , mdiArea(new QMdiArea(parent))
         , windowMapper(new QSignalMapper(parent))
         , documentMapper(new QSignalMapper(parent))
-        , lastExportSpecialOutputFlag(0)
     {
     }
 
@@ -225,12 +230,14 @@ public:
     KActionMenu *dockWidgetMenu;
     KActionMenu *windowMenu;
     KActionMenu *documentMenu;
+    KActionMenu *workspaceMenu;
 
     KHelpMenu *helpMenu  {0};
 
     KRecentFilesAction *recentFiles {0};
+    KoResourceModel *workspacemodel {0};
 
-    QUrl lastExportUrl;
+    QUrl lastExportLocation;
 
     QMap<QString, QDockWidget *> dockWidgetsMap;
     QMap<QDockWidget *, bool> dockWidgetVisibilityMap;
@@ -247,7 +254,6 @@ public:
     QSignalMapper *documentMapper;
 
     QByteArray lastExportedFormat;
-    int lastExportSpecialOutputFlag  {0};
     QScopedPointer<KisSignalCompressorWithParam<int> > tabSwitchCompressor;
     QMutex savingEntryMutex;
 
@@ -271,6 +277,11 @@ KisMainWindow::KisMainWindow()
     : KXmlGuiWindow()
     , d(new Private(this))
 {
+    auto rserver = KisResourceServerProvider::instance()->workspaceServer(false);
+    QSharedPointer<KoAbstractResourceServerAdapter> adapter(new KoResourceServerAdapter<KisWorkspaceResource>(rserver));
+    d->workspacemodel = new KoResourceModel(adapter, this);
+    connect(d->workspacemodel, &KoResourceModel::afterResourcesLayoutReset, this, [&]() { updateWindowMenu(); });
+
     KisConfig cfg;
 
     d->viewManager = new KisViewManager(this, actionCollection());
@@ -530,6 +541,9 @@ void KisMainWindow::addView(KisView *view)
         d->activeView->disconnect(this);
     }
 
+    // register the newly created view in the input manager
+    viewManager()->inputManager()->addTrackedCanvas(view->canvasBase());
+
     showView(view);
     updateCaption();
     emit restoringDone();
@@ -539,6 +553,15 @@ void KisMainWindow::addView(KisView *view)
     }
 }
 
+void KisMainWindow::notifyChildViewDestroyed(KisView *view)
+{
+    viewManager()->inputManager()->removeTrackedCanvas(view->canvasBase());
+    if (view->canvasBase() == viewManager()->canvasBase()) {
+        viewManager()->setCurrentView(0);
+    }
+}
+
+
 void KisMainWindow::showView(KisView *imageView)
 {
     if (imageView && activeView() != imageView) {
@@ -906,8 +929,8 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
     if (document->url().isEmpty()) {
         reset_url = true;
         saveas = true;
-    } else {
-        reset_url = false;
+    }
+    else { reset_url = false;
     }
 
     connect(document, SIGNAL(sigProgress(int)), this, SLOT(slotProgress(int)));
@@ -920,45 +943,21 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
     QByteArray _native_format = document->nativeFormatMimeType();
     QByteArray oldOutputFormat = document->outputMimeType();
 
-    int oldSpecialOutputFlag = document->specialOutputFlag();
-
     QUrl suggestedURL = document->url();
 
-    QStringList mimeFilter;
+    QStringList mimeFilter = KisImportExportManager::mimeFilter(KisImportExportManager::Export);
 
-    if (specialOutputFlag) {
-        mimeFilter = KisMimeDatabase::suffixesForMimeType(_native_format);
-    }
-    else {
-        mimeFilter = KisImportExportManager::mimeFilter(KisImportExportManager::Export);
-    }
-
-
-    if (!mimeFilter.contains(oldOutputFormat) && !isExporting()) {
+    if (!mimeFilter.contains(oldOutputFormat)) {
         dbgUI << "KisMainWindow::saveDocument no export filter for" << oldOutputFormat;
 
         // --- don't setOutputMimeType in case the user cancels the Save As
         // dialog and then tries to just plain Save ---
 
         // suggest a different filename extension (yes, we fortunately don't all live in a world of magic :))
-        QString suggestedFilename = suggestedURL.fileName();
+        QString suggestedFilename = QFileInfo(suggestedURL.toLocalFile()).baseName();
 
         if (!suggestedFilename.isEmpty()) {  // ".kra" looks strange for a name
-            int c = suggestedFilename.lastIndexOf('.');
-
-            const QString ext = KisMimeDatabase::suffixesForMimeType(_native_format).first();
-            if (!ext.isEmpty()) {
-                if (c < 0)
-                    suggestedFilename = suggestedFilename + "." + ext;
-                else
-                    suggestedFilename = suggestedFilename.left(c) + "." + ext;
-            } else { // current filename extension wrong anyway
-                if (c > 0) {
-                    // this assumes that a . signifies an extension, not just a .
-                    suggestedFilename = suggestedFilename.left(c);
-                }
-            }
-
+            suggestedFilename = suggestedFilename + "." + KisMimeDatabase::suffixesForMimeType(_native_format).first().remove("*.");
             suggestedURL = suggestedURL.adjusted(QUrl::RemoveFilename);
             suggestedURL.setPath(suggestedURL.path() + suggestedFilename);
         }
@@ -969,21 +968,53 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
 
     bool ret = false;
 
-    if (document->url().isEmpty() || saveas) {
+    if (document->url().isEmpty() || d->isExporting || saveas) {
         // if you're just File/Save As'ing to change filter options you
         // don't want to be reminded about overwriting files etc.
         bool justChangingFilterOptions = false;
 
         KoFileDialog dialog(this, KoFileDialog::SaveFile, "SaveAs");
-        dialog.setCaption(i18n("untitled"));
-        if (isExporting() && !d->lastExportUrl.isEmpty()) {
-            dialog.setDefaultDir(d->lastExportUrl.toLocalFile());
+        dialog.setCaption(d->isExporting ? i18n("Exporting") : i18n("Saving As"));
+
+        if (d->isExporting && !d->lastExportLocation.isEmpty()) {
+
+            // Use the location where we last exported to, if it's set, as the opening location for the file dialog
+            QString proposedPath = QFileInfo(d->lastExportLocation.toLocalFile()).absolutePath();
+            // If the document doesn't have a filename yet, use the title
+            QString proposedFileName = suggestedURL.isEmpty() ? document->documentInfo()->aboutInfo("title") :  QFileInfo(suggestedURL.toLocalFile()).baseName();
+            // Use the last mimetype we exported to by default
+            QString proposedMimeType =  d->lastExportedFormat.isEmpty() ? "" : d->lastExportedFormat;
+            QString proposedExtension = KisMimeDatabase::suffixesForMimeType(proposedMimeType).first().remove("*,");
+
+            // Set the default dir: this overrides the one loaded from the config file, since we're exporting and the lastExportLocation is not empty
+            dialog.setDefaultDir(proposedPath + "/" + proposedFileName + "." + proposedExtension, true);
+            dialog.setMimeTypeFilters(mimeFilter, proposedMimeType);
         }
         else {
-            dialog.setDefaultDir(suggestedURL.toLocalFile());
+            // Get the last used location for saving
+            KConfigGroup group =  KSharedConfig::openConfig()->group("File Dialogs");
+            QString proposedPath = group.readEntry("SaveAs", "");
+            // if that is empty, get the last used location for loading
+            if (proposedPath.isEmpty()) {
+                proposedPath = group.readEntry("OpenDocument", "");
+            }
+            // If that is empty, too, use the Pictures location.
+            if (proposedPath.isEmpty()) {
+                proposedPath = QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
+            }
+            // But only use that if the suggestedUrl, that is, the document's own url is empty, otherwise
+            // open the location where the document currently is.
+            dialog.setDefaultDir(suggestedURL.isEmpty() ? proposedPath : suggestedURL.toLocalFile(), true);
+
+            // If exporting, default to all supported file types if user is exporting
+            QByteArray default_mime_type = "";
+            if (!d->isExporting) {
+                // otherwise use the document's mimetype, or if that is empty, kra, which is the savest.
+                default_mime_type = document->outputMimeType().isEmpty() ? _native_format : document->outputMimeType();
+            }
+            dialog.setMimeTypeFilters(mimeFilter, QString::fromLatin1(default_mime_type));
         }
-        // Default to all supported file types if user is exporting, otherwise use Krita default
-        dialog.setMimeTypeFilters(mimeFilter, QString(_native_format));
+
         QUrl newURL = QUrl::fromUserInput(dialog.filename());
 
         if (newURL.isLocalFile()) {
@@ -1002,19 +1033,17 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
 
         QByteArray outputFormat = _native_format;
 
-        if (!specialOutputFlag) {
-            QString outputFormatString = KisMimeDatabase::mimeTypeForFile(newURL.toLocalFile());
-            outputFormat = outputFormatString.toLatin1();
-        }
+        QString outputFormatString = KisMimeDatabase::mimeTypeForFile(newURL.toLocalFile());
+        outputFormat = outputFormatString.toLatin1();
 
-        if (!isExporting())
+        if (!isExporting()) {
             justChangingFilterOptions = (newURL == document->url()) &&
-                    (outputFormat == document->mimeType()) &&
-                    (specialOutputFlag == oldSpecialOutputFlag);
-        else
-            justChangingFilterOptions = (newURL == d->lastExportUrl) &&
-                    (outputFormat == d->lastExportedFormat) &&
-                    (specialOutputFlag == d->lastExportSpecialOutputFlag);
+                    (outputFormat == document->mimeType());
+        }
+        else {
+            justChangingFilterOptions = (newURL == d->lastExportLocation) &&
+                    (outputFormat == d->lastExportedFormat);
+        }
 
 
         bool bOk = true;
@@ -1022,14 +1051,6 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
             bOk = false;
         }
 
-        // adjust URL before doing checks on whether the file exists.
-        if (specialOutputFlag) {
-            QString fileName = newURL.fileName();
-            if ( specialOutputFlag== KisDocument::SaveAsDirectoryStore) {
-                //dbgKrita << "save to directory: " << newURL.url();
-            }
-        }
-
         if (bOk) {
             bool wantToSave = true;
 
@@ -1063,7 +1084,7 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
                 //
                 // - Clarence
                 //
-                document->setOutputMimeType(outputFormat, specialOutputFlag);
+                document->setOutputMimeType(outputFormat);
                 if (!isExporting()) {  // Save As
                     ret = document->saveAs(newURL);
 
@@ -1075,20 +1096,19 @@ bool KisMainWindow::saveDocument(KisDocument *document, bool saveas, bool silent
                         dbgUI << "Failed Save As!";
                         document->setUrl(oldURL);
                         document->setLocalFilePath(oldFile);
-                        document->setOutputMimeType(oldOutputFormat, oldSpecialOutputFlag);
+                        document->setOutputMimeType(oldOutputFormat);
                     }
                 } else { // Export
                     ret = document->exportDocument(newURL);
 
                     if (ret) {
                         // a few file dialog convenience things
-                        d->lastExportUrl = newURL;
+                        d->lastExportLocation = newURL;
                         d->lastExportedFormat = outputFormat;
-                        d->lastExportSpecialOutputFlag = specialOutputFlag;
                     }
 
                     // always restore output format
-                    document->setOutputMimeType(oldOutputFormat, oldSpecialOutputFlag);
+                    document->setOutputMimeType(oldOutputFormat);
                 }
 
                 if (silent) // don't let the document change the window caption
@@ -2070,6 +2090,73 @@ void KisMainWindow::updateWindowMenu()
         }
     }
 
+    menu->addAction(d->workspaceMenu);
+    QMenu *workspaceMenu = d->workspaceMenu->menu();
+    workspaceMenu->clear();
+
+    auto workspaces = KisResourceServerProvider::instance()->workspaceServer(false)->resources();
+    auto m_this = this;
+    for (auto &w : workspaces) {
+        auto action = workspaceMenu->addAction(w->name());
+        auto ds = w->dockerState();
+        connect(action, &QAction::triggered, this, [=]() { m_this->restoreWorkspace(ds); });
+    }
+    workspaceMenu->addSeparator();
+    connect(workspaceMenu->addAction(i18nc("@action:inmenu", "&Import Workspace...")),
+            &QAction::triggered,
+            this,
+            [&]() {
+        QString extensions = d->workspacemodel->extensions();
+        QStringList mimeTypes;
+        for(const QString &suffix : extensions.split(":")) {
+            mimeTypes << KisMimeDatabase::mimeTypeForSuffix(suffix);
+        }
+
+        KoFileDialog dialog(0, KoFileDialog::OpenFile, "OpenDocument");
+        dialog.setMimeTypeFilters(mimeTypes);
+        dialog.setCaption(i18nc("@title:window", "Choose File to Add"));
+        QString filename = dialog.filename();
+
+        d->workspacemodel->importResourceFile(filename);
+    });
+
+    connect(workspaceMenu->addAction(i18nc("@action:inmenu", "&New Workspace...")),
+            &QAction::triggered,
+            [=]() {
+        QString name = QInputDialog::getText(this, i18nc("@title:window", "New Workspace..."),
+                                             i18nc("@label:textbox", "Name:"));
+        if (name.isEmpty()) return;
+        auto rserver = KisResourceServerProvider::instance()->workspaceServer();
+
+        KisWorkspaceResource* workspace = new KisWorkspaceResource("");
+        workspace->setDockerState(m_this->saveState());
+        d->viewManager->resourceProvider()->notifySavingWorkspace(workspace);
+        workspace->setValid(true);
+        QString saveLocation = rserver->saveLocation();
+
+        bool newName = false;
+        if(name.isEmpty()) {
+            newName = true;
+            name = i18n("Workspace");
+        }
+        QFileInfo fileInfo(saveLocation + name + workspace->defaultFileExtension());
+
+        int i = 1;
+        while (fileInfo.exists()) {
+            fileInfo.setFile(saveLocation + name + QString("%1").arg(i) + workspace->defaultFileExtension());
+            i++;
+        }
+        workspace->setFilename(fileInfo.filePath());
+        if(newName) {
+            name = i18n("Workspace %1", i);
+        }
+        workspace->setName(name);
+        rserver->addResource(workspace);
+    });
+
+    // TODO: What to do about delete?
+//    workspaceMenu->addAction(i18nc("@action:inmenu", "&Delete Workspace..."));
+
     menu->addSeparator();
     menu->addAction(d->close);
     menu->addAction(d->closeAll);
@@ -2133,6 +2220,27 @@ void KisMainWindow::configChanged()
     Q_FOREACH (QMdiSubWindow *subwin, d->mdiArea->subWindowList()) {
         subwin->setOption(QMdiSubWindow::RubberBandMove, cfg.readEntry<int>("mdi_rubberband", cfg.useOpenGL()));
         subwin->setOption(QMdiSubWindow::RubberBandResize, cfg.readEntry<int>("mdi_rubberband", cfg.useOpenGL()));
+
+        /**
+         * Dirty workaround for a bug in Qt (checked on Qt 5.6.1):
+         *
+         * If you make a window "Show on top" and then switch to the tabbed mode
+         * the window will contiue to be painted in its initial "mid-screen"
+         * position. It will persist here until you explicitly switch to its tab.
+         */
+        if (viewMode == QMdiArea::TabbedView) {
+            Qt::WindowFlags oldFlags = subwin->windowFlags();
+            Qt::WindowFlags flags = oldFlags;
+
+            flags &= ~Qt::WindowStaysOnTopHint;
+            flags &= ~Qt::WindowStaysOnBottomHint;
+
+            if (flags != oldFlags) {
+                subwin->setWindowFlags(flags);
+                subwin->showMaximized();
+            }
+        }
+
     }
 
     KConfigGroup group( KSharedConfig::openConfig(), "theme");
