@@ -57,6 +57,9 @@
 #include "kis_guides_config.h"
 #include "KisProofingConfiguration.h"
 
+#include <QFileInfo>
+#include <QDir>
+
 
 using namespace KRA;
 
@@ -126,6 +129,7 @@ QDomElement KisKraSaver::saveXML(QDomDocument& doc,  KisImageWSP image)
     saveAssistantsList(doc,imageElement);
     saveGrid(doc,imageElement);
     saveGuides(doc,imageElement);
+    saveAudio(doc,imageElement);
 
     QDomElement animationElement = doc.createElement("animation");
     KisDomUtils::saveValue(&animationElement, "framerate", image->animationInterface()->framerate());
@@ -418,3 +422,31 @@ bool KisKraSaver::saveGuides(QDomDocument& doc, QDomElement& element)
     return true;
 }
 
+bool KisKraSaver::saveAudio(QDomDocument& doc, QDomElement& element)
+{
+    const KisImageAnimationInterface *interface = m_d->doc->image()->animationInterface();
+    QString fileName = interface->audioChannelFileName();
+    if (fileName.isEmpty()) return true;
+
+    if (!QFileInfo::exists(fileName)) {
+        m_d->errorMessages << i18n("Audio channel file %1 doesn't exist!", fileName);
+        return false;
+    }
+
+    const QDir documentDir = QFileInfo(m_d->doc->localFilePath()).absoluteDir();
+    KIS_ASSERT_RECOVER_RETURN_VALUE(documentDir.exists(), false);
+
+    fileName = documentDir.relativeFilePath(fileName);
+    fileName = QDir::fromNativeSeparators(fileName);
+
+    KIS_ASSERT_RECOVER_RETURN_VALUE(!fileName.isEmpty(), false);
+
+    QDomElement audioElement = doc.createElement("audio");
+    KisDomUtils::saveValue(&audioElement, "masterChannelPath", fileName);
+    KisDomUtils::saveValue(&audioElement, "audioMuted", interface->isAudioMuted());
+    KisDomUtils::saveValue(&audioElement, "audioVolume", interface->audioVolume());
+    element.appendChild(audioElement);
+
+    return true;
+}
+
