@@ -15,6 +15,10 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <kconfig.h>
+#include <kconfiggroup.h>
+#include <ksharedconfig.h>
+
 #include "imagedocker_dock.h"
 #include "image_strip_scene.h"
 #include "image_view.h"
@@ -48,7 +52,7 @@
 
 class ImageFilter: public QSortFilterProxyModel
 {
-    virtual bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const {
+    bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const override {
         QFileSystemModel* model = static_cast<QFileSystemModel*>(sourceModel());
         QModelIndex       index = sourceModel()->index(source_row, 0, source_parent);
 
@@ -72,7 +76,7 @@ class ImageFilter: public QSortFilterProxyModel
         return false;
     }
 
-    virtual bool filterAcceptsColumn(int source_column, const QModelIndex& source_parent) const {
+    bool filterAcceptsColumn(int source_column, const QModelIndex& source_parent) const override {
         Q_UNUSED(source_parent);
         return source_column == 0;
     }
@@ -128,12 +132,12 @@ public:
         return -1;
     }
 
-    virtual int rowCount(const QModelIndex& parent=QModelIndex()) const {
+    int rowCount(const QModelIndex& parent=QModelIndex()) const override {
         Q_UNUSED(parent);
         return m_data.size();
     }
 
-    virtual QVariant data(const QModelIndex& index, int role=Qt::DisplayRole) const {
+    QVariant data(const QModelIndex& index, int role=Qt::DisplayRole) const override {
         if(index.isValid() && index.row() < m_data.size()) {
             switch(role)
             {
@@ -224,8 +228,7 @@ ImageDockerDock::ImageDockerDock():
 
     connect(m_ui->cmbPath, SIGNAL(activated(const QString&)), SLOT(slotChangeRoot(const QString&)));
 
-    m_model->setRootPath(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation));
-    m_ui->treeView->setRootIndex(m_proxyModel->mapFromSource(m_model->index(QDesktopServices::storageLocation(QDesktopServices::PicturesLocation))));
+    loadConfigState();
 
     connect(m_ui->treeView           , SIGNAL(doubleClicked(const QModelIndex&))      , SLOT(slotItemDoubleClicked(const QModelIndex&)));
     connect(m_ui->bnBack             , SIGNAL(clicked(bool))                          , SLOT(slotBackButtonClicked()));
@@ -249,6 +252,8 @@ ImageDockerDock::ImageDockerDock():
 
 ImageDockerDock::~ImageDockerDock()
 {
+    saveConfigState();
+
     delete m_proxyModel;
     delete m_model;
     delete m_imageStripScene;
@@ -374,6 +379,30 @@ void ImageDockerDock::setZoom(const ImageInfo& info)
     m_popupUi->zoomSlider->blockSignals(false);
 }
 
+void ImageDockerDock::saveConfigState()
+{
+    const QString lastUsedDirectory = m_model->filePath(m_proxyModel->mapToSource(m_ui->treeView->rootIndex()));
+
+    KConfigGroup cfg =  KSharedConfig::openConfig()->group("referenceImageDocker");
+    cfg.writeEntry("lastUsedDirectory", lastUsedDirectory);
+}
+
+void ImageDockerDock::loadConfigState()
+{
+    const QString defaultLocation = QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
+
+    KConfigGroup cfg =  KSharedConfig::openConfig()->group("referenceImageDocker");
+    QString lastUsedDirectory = cfg.readEntry("lastUsedDirectory", defaultLocation);
+
+    if (!QFileInfo(lastUsedDirectory).exists()) {
+        lastUsedDirectory = defaultLocation;
+    }
+
+    m_model->setRootPath(lastUsedDirectory);
+    m_ui->treeView->setRootIndex(m_proxyModel->mapFromSource(m_model->index(lastUsedDirectory)));
+    updatePath(lastUsedDirectory);
+}
+
 
 // ------------ slots ------------------------------------------------- //
 
