@@ -26,6 +26,7 @@
 
 #include "KisMainWindow.h"
 #include "KisViewManager.h"
+#include "kis_transform_utils.h"
 
 #include "kis_transform_utils.h"
 
@@ -148,6 +149,7 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
 
 
     connect(m_rotationCenterButtons, SIGNAL(buttonPressed(int)), this, SLOT(slotRotationCenterChanged(int)));
+    connect(btnTransformAroundPivotPoint, SIGNAL(clicked(bool)), this, SLOT(slotTransformAroundRotationCenter(bool)));
 
     // Init Free Transform Values
     connect(scaleXBox, SIGNAL(valueChanged(int)), this, SLOT(slotSetScaleX(int)));
@@ -200,12 +202,12 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     liquifySizeSlider->setRange(KisLiquifyProperties::minSize(),
                                 KisLiquifyProperties::maxSize(), 2);
     liquifySizeSlider->setExponentRatio(4);
-    liquifySizeSlider->setValue(50.0);
+    liquifySizeSlider->setValue(60.0);
     connect(liquifySizeSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifySizeChanged(qreal)));
     liquifySizeSlider->setToolTip(i18nc("@info:tooltip", "Size of the deformation brush"));
 
     liquifyAmountSlider->setRange(0.0, 1.0, 2);
-    liquifyAmountSlider->setValue(1.0);
+    liquifyAmountSlider->setValue(0.05);
     connect(liquifyAmountSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifyAmountChanged(qreal)));
     liquifyAmountSlider->setToolTip(i18nc("@info:tooltip", "Amount of the deformation you get"));
 
@@ -214,12 +216,10 @@ KisToolTransformConfigWidget::KisToolTransformConfigWidget(TransformTransactionP
     connect(liquifyFlowSlider, SIGNAL(valueChanged(qreal)), this, SLOT(liquifyFlowChanged(qreal)));
     liquifyFlowSlider->setToolTip(i18nc("@info:tooltip", "When in non-buildup mode, shows how fast the deformation limit is reached."));
 
-
     buidupModeComboBox->setCurrentIndex(0); // set to build-up mode by default
     connect(buidupModeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(liquifyBuildUpChanged(int)));
     buidupModeComboBox->setToolTip(i18nc("@info:tooltip", "Switch between Build Up and Wash mode of painting. Build Up mode adds deformations one on top of the other without any limits. Wash mode gradually deforms the piece to the selected deformation level."));
 
-
     liquifySpacingSlider->setRange(0.0, 3.0, 2);
     liquifySizeSlider->setExponentRatio(3);
     liquifySpacingSlider->setSingleStep(0.01);
@@ -324,6 +324,8 @@ void KisToolTransformConfigWidget::slotUpdateIcons()
     middleBottomButton->setIcon(KisIconUtils::loadIcon("arrow-down"));
     bottomRightButton->setIcon(KisIconUtils::loadIcon("arrow-downright"));
 
+    btnTransformAroundPivotPoint->setIcon(KisIconUtils::loadIcon("pivot-point"));
+
 
     // pressure icons
     liquifySizePressureBox->setIcon(KisIconUtils::loadIcon("transform_icons_penPressure"));
@@ -573,6 +575,9 @@ void KisToolTransformConfigWidget::updateConfig(const ToolTransformArgs &config)
                 break;
             }
         }
+
+        btnTransformAroundPivotPoint->setChecked(config.transformAroundRotationCenter());
+
     } else if (config.mode() == ToolTransformArgs::WARP) {
 
         stackedWidget->setCurrentIndex(1);
@@ -846,12 +851,26 @@ void KisToolTransformConfigWidget::slotRotationCenterChanged(int index)
 
 }
 
+void KisToolTransformConfigWidget::slotTransformAroundRotationCenter(bool value)
+{
+    if (m_uiSlotsBlocked) return;
+
+    ToolTransformArgs *config = m_transaction->currentConfig();
+    config->setTransformAroundRotationCenter(value);
+    notifyConfigChanged();
+    notifyEditingFinished();
+}
+
 void KisToolTransformConfigWidget::slotSetScaleX(int value)
 {
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleX(value / 100.);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleX(value / 100.);
+    }
 
     if (config->keepAspectRatio()) {
 
@@ -875,7 +894,11 @@ void KisToolTransformConfigWidget::slotSetScaleY(int value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleY(value / 100.);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleY(value / 100.);
+    }
 
     if (config->keepAspectRatio()) {
         blockNotifications();
@@ -896,7 +919,12 @@ void KisToolTransformConfigWidget::slotSetShearX(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setShearX((double)value);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setShearX((double)value);
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -906,7 +934,13 @@ void KisToolTransformConfigWidget::slotSetShearY(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setShearY((double)value);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setShearY((double)value);
+    }
+
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -948,7 +982,10 @@ void KisToolTransformConfigWidget::slotSetAX(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAX(degreeToRadian((double)value));
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAX(degreeToRadian((double)value));
+    }
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -958,7 +995,12 @@ void KisToolTransformConfigWidget::slotSetAY(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAY(degreeToRadian((double)value));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAY(degreeToRadian((double)value));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -968,7 +1010,12 @@ void KisToolTransformConfigWidget::slotSetAZ(qreal value)
     if (m_uiSlotsBlocked) return;
 
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAZ(degreeToRadian((double)value));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(degreeToRadian((double)value));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -976,7 +1023,12 @@ void KisToolTransformConfigWidget::slotSetAZ(qreal value)
 void KisToolTransformConfigWidget::slotFlipX()
 {
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleX(config->scaleX() * -1);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleX(config->scaleX() * -1);
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -984,7 +1036,12 @@ void KisToolTransformConfigWidget::slotFlipX()
 void KisToolTransformConfigWidget::slotFlipY()
 {
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setScaleY(config->scaleY() * -1);
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setScaleY(config->scaleY() * -1);
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -992,7 +1049,12 @@ void KisToolTransformConfigWidget::slotFlipY()
 void KisToolTransformConfigWidget::slotRotateCW()
 {
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAZ(normalizeAngle(config->aZ() + M_PI_2));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(normalizeAngle(config->aZ() + M_PI_2));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
@@ -1000,7 +1062,12 @@ void KisToolTransformConfigWidget::slotRotateCW()
 void KisToolTransformConfigWidget::slotRotateCCW()
 {
     ToolTransformArgs *config = m_transaction->currentConfig();
-    config->setAZ(normalizeAngle(config->aZ() - M_PI_2));
+
+    {
+        KisTransformUtils::AnchorHolder keeper(config->transformAroundRotationCenter(), config);
+        config->setAZ(normalizeAngle(config->aZ() - M_PI_2));
+    }
+
     notifyConfigChanged();
     notifyEditingFinished();
 }
